// UDP_Client_Service class implementation file

/*
  Currently there's a problem reading in the message data as a string. It only works
  if you read it as it's true data type or if you have a string constant...
*/

#include "UDP_Client_Service.h"
#include <cstring>



UDP_Client_Service::UDP_Client_Service(std::string remote_ip, std::string remote_port, std::string current_ip, std::string listening_port)
{
  local_ip = current_ip;
  local_port = listening_port;
  
  //i figured it might be nice to keep the server info in memory...
  server_ip = remote_ip;
  server_port = remote_port;

  printf("\nInitializing SDL_net (sending)...");
  if (SDLNet_Init() < 0)
  {
    fprintf(stderr, "FAILED!\nSDLNet_Init: %s\n", SDLNet_GetError());
    exit(1);
  }
  printf("Success!\n");
  
  //resolve server address
  resolve_server_address();
  
  //open socket on random port
  printf("Opening random communication socket for sending...");
  if (!(sd_send = SDLNet_UDP_Open(0)))
  {
    fprintf(stderr, "FAILED!\nSDLNet_UDP_Open(0): %s\n", SDLNet_GetError());
    exit(1);
  }
  printf("Success!\n");
    
  //allocate a vector of 3 packets of 512b in memory for sending packets.
  printf("Allocating memory for sending packets...");
  if ((p_send = SDLNet_AllocPacketV(MAX_PACKETS, 1024)) == NULL)
  {
    fprintf(stderr, "FAILED!\nSDLNet_AllocPacketV(3, 1024): %s\n", SDLNet_GetError());
    exit(1);
  }
  printf("Success!\n");

  
}


UDP_Client_Service::~UDP_Client_Service()
{
    SDLNet_UDP_Close(sd_send);

    printf("Shutting down sending client...\n");
    //printf("Freeing allocated memory for outbound packets...");
    //SDLNet_FreePacketV(p_send);
    printf("Done!\nExiting SDLNet and terminating server...");
    SDLNet_Quit();
    printf("Done!\n");

}

void UDP_Client_Service::send_logon(void)
{
  //sends logon information. 
  //basically a message w/ "NEW_USER" as the data (with IP & port of course)
}

void UDP_Client_Service::send_msg(std::string message)
{
  int i;
  std::string message1 = "po";

  Uint32 data_to_send[3];
  
  data_to_send[IP] = atoi(message1.c_str());
  std::cout << "data_to_send[IP]: " << data_to_send[IP] << std::endl;
      //filling packets information...

  //04-04-18: Fixing so message is set correctly again.
  /*
  SDLNet_Write32(data_to_send[IP], p_send[IP]->data);
  SDLNet_Write16(2, p_send[PORT]->data);
  SDLNet_Write16(3, p_send[MSG]->data);
  */

  p_send[IP]->data = (Uint8*)server_ip.c_str();
  p_send[PORT]->data = (Uint8*)server_port.c_str();
  p_send[MSG]->data = (Uint8*)message.c_str();

  
//  cout << "  p_send[IP]->data = " << (char *)SDLNet_Read32(p_send[IP]->data) << endl; 
 // cout << "p_send[PORT]->data = " << (char *)SDLNet_Read32(p_send[PORT]->data) << endl; 
  //cout << " p_send[MSG]->data = " << (char *)SDLNet_Read32(p_send[MSG]->data) << endl << endl; 
  
    //set destination host & port
    printf("Setting packet vector destination info...\n");
    for (i = 0; i < MAX_PACKETS; i++)
    {    
      std::cout << "Packet[" << i << "]: setting host->";
      p_send[i]->address.host = server_address.host;
      std::cout << "setting port->";
      p_send[i]->address.port = server_address.port;
      std::cout << "setting length->";
      p_send[i]->len = sizeof(p_send[i]->data) + 1; //strlen((const char *)SDLNet_Read32(p_send[i]->data)) + 1
      std::cout << p_send[i]->len << "setting channel...\n";
      p_send[i]->channel = -1;
    }



    //send the packet
    printf("Sending...\n");
    if((SDLNet_UDP_SendV(sd_send, p_send, MAX_PACKETS)) == -1)  //socket, packet, # of packets.
      std::cout << "Error sending packet!\n";
     

}


void UDP_Client_Service::resolve_server_address(void)
{
  
  printf("Resolving server address (%s [%s])...", server_ip.c_str(), server_port.c_str());

  if ((SDLNet_ResolveHost(&server_address, server_ip.c_str(), static_cast<Uint16>(atoi(server_port.c_str())))) != 0)
  {
    fprintf(stderr, "FAILED!\nSDLNet_ResolveHost(%s %s): %s\n", server_ip.c_str(), server_port.c_str(), SDLNet_GetError());
    exit(1);
  }
  
  else 
    std::cout << "Success!\n";
  
}


void UDP_Client_Service::show_dest_info(void) const
{
  printf("Sending packets to %s on port %s\n", server_ip.c_str(), server_port.c_str());
}
