
/*
UDP Client
-----------

Client side

   1. Initialize SDL_net library
   2. Open a socket on a random usable port
   3. Resolve the server address
   4. Allocate memory for packets
   5. Fill and send the packet
   6. Free the memory and quit 
   
   currently converting all network programming to single "UDP_Service" class so code is neater
   for game programming.
*/

#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <iostream>
#include "UDP_Server_Service.h"
#include "UDP_Client_Service.h"

//#include "player.h"
#include "Get_Key.h"


int checkArgs(int, char**);   //checks that valid command args were sent to program
void displayHelp(void);

int main(int argc, char *argv[])
{
  int quit = 0;
  std::string msg;
  std::string cmd;

  
  if (!checkArgs(argc, argv))
  {
    fprintf(stderr, "\nInvalid syntax!\n\tupd_client <other_ip> <other_ip's_send_port> <listen_port>\n\n");
    exit(1);
  }
  system("clear");

  std::string server_ip = (const char*)argv[1];
  std::string server_port = (const char *)argv[2];
  std::string local_ip = (const char *)argv[3];
  std::string listen_port = (const char *)argv[4];

  UDP_Client_Service client(server_ip, server_port, local_ip, listen_port); //client obj deals ONLY w/ outbound!
  UDP_Server_Service server(local_ip, listen_port);  //server obj deals ONLY w/ inbound!

  //main loop
  while (!quit)
  {
    
    if (kbhit())
    {
      if (getch() == '\n')
      {
        printf("\n$ "); 
        std::cin >> cmd;
  
        if (cmd == "send")
        {
          printf(" > ");
	  std::cin >> msg;
          client.send_msg(msg);
    
          //if packet data contains "quit" do so.
          if (msg == "quit")
          {
            //system("clear"); //04-04-18: Don't clear screen on quit.
            printf("msg = %s\n", msg.c_str());
            printf("\nClient sent \"quit\" command!\n");
            quit = 1;
          }
        }
	
        else if (cmd == "help")
          displayHelp();
      
        else if (cmd == "exit")
          quit = 1;
	  
	
	else if (cmd == "show_dest_info")
	  client.show_dest_info();
	  
      }
      
    }
    
    if (!server.recieve_data())
      quit = 1;
      
  }
  
  printf("\nTerminating program normally...\n");

  return 0;
  
}

int checkArgs(int a_count, char *a_variable[])
{
  std::string choice;
 
  if (a_count != 5)
    return 0;
  
  printf("\n   IP Address to connect to: %s", a_variable[1]);
  printf("\n  Port number to connect to: %s", a_variable[2]);
  printf("\n     IP address to reply to: %s", a_variable[3]);
  printf("\nListen port to recieve from: %s", a_variable[4]);
  
  printf("\nIs this correct? [Y/n] ");
  //std::cin >> choice;
  
  //if ((choice == "n") || (choice == "N"))
  //  return 0;
  //else
    return 1;
}

void displayHelp(void)
{
  printf("\n Help:\n");
  printf("\tshow_dest_info - displays current destination IP address and port number\n");
  printf("\t          send - send data packet to destination IP address at selected port\n");
  printf("\t          exit - quit program, but don't force other client to quit too\n");
  printf("\t          help - display this message\n\n");
}
